
from django.core.management.base import BaseCommand
from nacl_encrypted_fields.backends import NaClWrapper


class Command(BaseCommand):
    help = 'Create a key for NaClWrapper.'

    def add_arguments(self, parser):
        parser.add_argument('-f', '--file', dest='filename',
                            help='Path to settings.py. The key will be '
                                 'written to this file automatically.')

    def handle(self, *args, **options):
        key = NaClWrapper.createKey()
        output = 'NACL_FIELDS_KEY = %s' % key

        filename = options['filename']
        if filename:
            with open(filename, 'a') as file:
                file.write('%s\n' % output)
        else:
            print('# put the following line in your settings.py\n%s' % output)




from nacl_encrypted_fields.backends.cryptowrapper import CryptoWrapper

from base64 import b85decode


# Test class that uses XOR to encrypt. Requirements are to implement encrypt()
# and decrypt(). NOTE: do not use this in production.
class TestCryptoWrapper(CryptoWrapper):
    def __init__(self, keydata, *args, **kwargs):
        self.key = bytearray(b85decode(keydata))

    def encrypt(self, plaintext):
        return bytes(bytearray([itr ^ self.key[idx % len(self.key)]
                     for idx, itr in enumerate(bytearray(plaintext))]))

    def decrypt(self, ciphertext):
        return bytes(bytearray([itr ^ self.key[idx % len(self.key)]
                     for idx, itr in enumerate(bytearray(ciphertext))]))




from django.db import models

from nacl_encrypted_fields.fields import (
    NaClBooleanField,
    NaClCharField,
    NaClDateField,
    NaClDateTimeField,
    NaClEmailField,
    NaClFloatField,
    NaClIntegerField,
    NaClTextField,
)

from tests.backends.testcryptowrapper import TestCryptoWrapper


class TestModel(models.Model):
    boolean = NaClBooleanField(default=False, blank=True)
    char = NaClCharField(max_length=255, null=True, blank=True)
    date = NaClDateField(null=True, blank=True)
    datetime = NaClDateTimeField(null=True, blank=True)
    email = NaClEmailField(null=True, blank=True)
    floating = NaClFloatField(null=True, blank=True)
    integer = NaClIntegerField(null=True, blank=True)
    text = NaClTextField(null=True, blank=True)

    custom_crypto_char = NaClCharField(max_length=255, null=True,
                                       crypto_class=TestCryptoWrapper,
                                       blank=True)




from nacl_encrypted_fields.backends.cryptowrapper import CryptoWrapper

import nacl.pwhash
import nacl.utils
import nacl.secret

from base64 import b85decode, b85encode
from django.core.exceptions import ImproperlyConfigured


class NaClWrapperException(ImproperlyConfigured):
    pass


# Simple wrapper around PyNaCl to standardise the initialization of the box
# object and allow for others to extend as needed.
class NaClWrapper(CryptoWrapper):
    def __init__(self, keydata, apply_kdf=False, *args, **kwargs):
        key = b85decode(keydata)
        if len(key) != nacl.secret.SecretBox.KEY_SIZE:
            raise NaClWrapperException('keysize must be equal to %d bytes',
                                       nacl.secret.SecretBox.KEY_SIZE)

        self.box = nacl.secret.SecretBox(key)

    def encrypt(self, plaintext):
        return self.box.encrypt(plaintext)

    def decrypt(self, ciphertext):
        return self.box.decrypt(ciphertext)

    @staticmethod
    def createKey():
        return b85encode(nacl.utils.random(nacl.secret.SecretBox.KEY_SIZE))




from django.core.management.base import BaseCommand

import base64


class Command(BaseCommand):
    help = 'Convert a v1.x to a v2.x key for NaClWrapper.'

    def add_arguments(self, parser):
        parser.add_argument('-k', '--key', dest='key',
                            help='v1.x key, base64 encoded', required=True)

    def handle(self, *args, **options):
        key = base64.b85encode(base64.b64decode(options['key']))
        output = 'NACL_FIELDS_KEY = %s' % key

        print('# replace the old line with the following line in your '
              'settings.py\n%s' % output)



import os
import re
import codecs
from setuptools import find_packages, setup


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# Allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-nacl-fields',
    version=find_version('nacl_encrypted_fields', '__init__.py'),
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    install_requires=[
        'Django>=2.2',
        'PyNaCl>=1.4.0',
    ],
    extras_require={
        'test': [
            'psycopg2-binary',
        ],
        'lint': [
            'flake8',
        ],
        'docs': [
            'sphinx',
        ]
    },
    license='Apache License',
    description=(
        'This is a collection of Django Model Field classes that are encrypted'
        ' using PyNaCl.'
    ),
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/warpnet/django-nacl-fields',
    author='Warpnet B.V.',
    author_email='info@warpnet.nl',
    classifiers=[
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.encoding import force_bytes, force_text
from django.utils.functional import cached_property

from nacl_encrypted_fields.backends.naclwrapper import NaClWrapper


class NaClFieldMixin(object):
    """
    NaClFieldMixin will use PyNaCl to encrypt/decrypt data that is
    being put in/out of the database into application Django model fields. This
    package is largely based on the django-encrypted-fields package, which
    makes use of the outdated Keyczar library to encrypt fields.

    The only way to use this mixin is to set the NACL_FIELDS_KEY in settings.py
    to a base64 encoded key that matches the key size of the crypto_class used.
    The default crypto class - NaCl's SecretBox - takes a 32 byte key.
    """

    def __init__(self, *args, **kwargs):
        """
        Initialize the NaClFieldMixin with the optional setting:
        * crypto_class: A custom class that is extended from CryptoWrapper.
        """
        crypto_class = kwargs.pop('crypto_class', NaClWrapper)

        key = settings.NACL_FIELDS_KEY
        if not key:
            raise ImproperlyConfigured(
                'You must set settings.NACL_FIELDS_KEY to use this library.'
            )

        self._crypto_box = crypto_class(key)

        super(NaClFieldMixin, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return 'BinaryField'

    def from_db_value(self, value, expression, connection, *args, **kwargs):
        if value is not None:
            value = bytes(value)
            if value != b'':
                value = self._crypto_box.decrypt(value)

            return super(NaClFieldMixin, self).to_python(force_text(value))

    def get_db_prep_save(self, value, connection):
        value = super(NaClFieldMixin, self).get_db_prep_save(value, connection)
        if value is not None:
            value = force_bytes(value)
            if value != b'':
                value = self._crypto_box.encrypt(value)

            return connection.Database.Binary(value)


class NaClBooleanField(NaClFieldMixin, models.BooleanField):
    pass


class NaClCharField(NaClFieldMixin, models.CharField):
    pass


class NaClDateTimeField(NaClFieldMixin, models.DateTimeField):
    pass


class NaClDateField(NaClFieldMixin, models.DateField):
    pass


class NaClEmailField(NaClFieldMixin, models.EmailField):
    pass


class NaClFloatField(NaClFieldMixin, models.FloatField):
    pass


class NaClIntegerField(NaClFieldMixin, models.IntegerField):
    @cached_property
    def validators(self):
        # Temporarily pretend to be an IntegerField to pass validation.
        # Changing `get_internal_type` on the fly to prevent fail in
        # Django>=1.7.
        self.get_internal_type = lambda: 'IntegerField'
        return models.IntegerField.validators.__get__(self)


class NaClTextField(NaClFieldMixin, models.TextField):
    pass
